// SPDX-FileCopyrightText: 2024 Joshua Goins <josh@redstrate.com>
// SPDX-License-Identifier: GPL-3.0-only

#include <KAboutData>
#include <KLocalizedString>

#include "InputDevice_interface.h"
#include "config.h"
#include "devices.h"
#include "ktabletconfig-version.h"
#include "parsing.h"

int main(int argc, char *argv[])
{
    QTextStream out(stdout);
    QTextStream err(stderr);

    QCoreApplication app(argc, argv);

    KLocalizedString::setApplicationDomain("ktabletconfig");

    KAboutData about(QStringLiteral("ktabletconfig"),
                     i18n("KDE Tablet Configurator"),
                     QString::fromUtf8(KTABLETCONFIG_VERSION_STRING),
                     i18n("Configure tablets from the CLI"),
                     KAboutLicense::GPL_V3,
                     i18n("© 2024 Joshua Goins"));
    about.addAuthor(i18n("Joshua Goins"), i18n("Maintainer"), QStringLiteral("josh@redstrate.com"));
    about.addCredit(i18n("Aleix Pol i Gonzalez"), i18n("Wrote kwin-inputdevice which this is based off of"));
    about.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
    KAboutData::setApplicationData(about);

    QCommandLineParser parser;

    const QCommandLineOption optionListDevices(QStringLiteral("list"), i18n("List connected tablets."));
    parser.addOption(optionListDevices);

    const QCommandLineOption tabletOption(QStringList() << QStringLiteral("tablet"), i18n("Set the name of the table to configure."), i18n("name"));
    parser.addOption(tabletOption);

    const QCommandLineOption buttonOption(QStringList() << QStringLiteral("button"),
                                          i18n("Set the index of the stylus button or pad button to configure."),
                                          i18n("index"));
    parser.addOption(buttonOption);

    const QCommandLineOption leftHandedOption(QStringList() << QStringLiteral("left-handed"), i18n("Query the left-handed mode of the tablet."));
    parser.addOption(leftHandedOption);

    const QCommandLineOption orientationOption(QStringList() << QStringLiteral("orientation"), i18n("Query the orientation of the tablet."));
    parser.addOption(orientationOption);

    const QCommandLineOption outputNameOption(QStringList() << QStringLiteral("output-name"), i18n("Query the output name of the tablet."));
    parser.addOption(outputNameOption);

    const QCommandLineOption stylusKey(QStringList() << QStringLiteral("stylus-button"), i18n("Queries the keybinding of the stylus button."), i18n("index"));
    parser.addOption(stylusKey);

    const QCommandLineOption padKey(QStringList() << QStringLiteral("pad-button"), i18n("Queries the keybinding of the pad button."), i18n("index"));
    parser.addOption(padKey);

    const QCommandLineOption setLeftHandedOption(QStringList() << QStringLiteral("set-left-handed"),
                                                 i18n("Sets the left-handed mode of the tablet."),
                                                 i18n("on/off"));
    parser.addOption(setLeftHandedOption);

    const QCommandLineOption setOrientationOption(QStringList() << QStringLiteral("set-orientation"),
                                                  i18n("Sets the orientation of the tablet."),
                                                  i18n("orientation"));
    parser.addOption(setOrientationOption);

    const QCommandLineOption setOutputNameOption(QStringList() << QStringLiteral("set-output-name"),
                                                 i18n("Sets the output name of the tablet."),
                                                 i18n("output-name"));
    parser.addOption(setOutputNameOption);

    const QCommandLineOption setStylusKey(QStringList() << QStringLiteral("set-stylus-button"),
                                          i18n("Sets the keybinding of the stylus button."),
                                          i18n("keybind"));
    parser.addOption(setStylusKey);

    const QCommandLineOption setPadKey(QStringList() << QStringLiteral("set-pad-button"), i18n("Sets the keybinding of the pad button."), i18n("keybind"));
    parser.addOption(setPadKey);

    about.setupCommandLine(&parser);
    parser.process(app);
    about.processCommandLine(&parser);

    const auto deviceSysNames = enumerateDevices();
    if (deviceSysNames.isEmpty()) {
        err << i18n("Failed to get list of tablets.") << Qt::endl;
        return -1;
    }

    if (parser.isSet(optionListDevices)) {
        for (const QString &sysname : deviceSysNames) {
            auto device = fetchDevice(sysname);
            if (device->tabletPad() || device->tabletTool()) {
                out << device->name() << Qt::endl;
            }
        }

        return 0;
    }

    if (!parser.isSet(tabletOption)) {
        err << i18n("Missing --tablet option, despite this option requiring it!") << Qt::endl;
        return -1;
    }

    QSharedPointer<OrgKdeKWinInputDeviceInterface> selectedDevice;

    for (const QString &sysname : deviceSysNames) {
        const auto device = fetchDevice(sysname);
        if (device->name() == parser.value(tabletOption)) {
            selectedDevice = device;
            break;
        }
    }

    if (!selectedDevice) {
        err << i18n("Invalid tablet name.") << Qt::endl;
        return -1;
    }

    if (parser.isSet(leftHandedOption)) {
        out << getBoolValue(selectedDevice->leftHanded()) << Qt::endl;
    }

    if (parser.isSet(orientationOption)) {
        if (const auto orientation = getOrientationOption(selectedDevice->orientationDBus())) {
            out << *orientation << Qt::endl;
        } else {
            return -1;
        }
    }

    if (parser.isSet(outputNameOption)) {
        out << selectedDevice->outputName() << Qt::endl;
    }

    if (parser.isSet(stylusKey)) {
        const auto button = getIntOption(parser.value(buttonOption));
        if (!button) {
            err << i18n("Invalid value for --button!") << Qt::endl;
            return -1;
        }
        out << readStylusKey(selectedDevice->name(), *button) << Qt::endl;
    }

    if (parser.isSet(padKey)) {
        const auto button = getIntOption(parser.value(buttonOption));
        if (!button) {
            err << i18n("Invalid value for --button!") << Qt::endl;
            return -1;
        }
        out << readPadKey(selectedDevice->name(), *button) << Qt::endl;
    }

    if (parser.isSet(setLeftHandedOption)) {
        const auto option = getBoolOption(parser.value(setLeftHandedOption));
        if (!option) {
            err << i18n("Invalid value for --set-left-handed!") << Qt::endl;
            return -1;
        }
        selectedDevice->setLeftHanded(*option);
    }

    if (parser.isSet(setOutputNameOption)) {
        const auto option = getOrientationOption(parser.value(setOutputNameOption));
        if (!option) {
            err << i18n("Invalid value for --set-output-name!") << Qt::endl;
            return -1;
        }
        selectedDevice->setOrientationDBus(*option);
    }

    if (parser.isSet(setOutputNameOption)) {
        const auto option = parser.value(setOutputNameOption);
        selectedDevice->setOutputName(option);
    }

    if (parser.isSet(setStylusKey)) {
        const auto key = parser.value(setStylusKey);
        const auto button = getIntOption(parser.value(buttonOption));
        if (!button) {
            err << i18n("Invalid value for --button!") << Qt::endl;
            return -1;
        }
        writeStylusKey(selectedDevice->name(), *button, key);
    }

    if (parser.isSet(setPadKey)) {
        const auto key = parser.value(setStylusKey);
        const auto button = getIntOption(parser.value(buttonOption));
        if (!button) {
            err << i18n("Invalid value for --button!") << Qt::endl;
            return -1;
        }
        writePadKey(selectedDevice->name(), *button, key);
    }

    return 0;
}